﻿Imports System.Text
Imports IdeaBlade.EntityModel
Imports DomainModel
Imports System.IO
Imports IdeaBlade.Core

Imports Crypto = IdeaBlade.Core.CryptoFns

Namespace Console01
  Friend Class Program
	Shared Sub Main(ByVal args() As String)
	  CType(New Program(), Program).DoIt()
	End Sub

	Private Sub DoIt()
	  LoadDataIntoCache()

	  Dim cacheFilePath As String = System.IO.Directory.GetCurrentDirectory() & "\devForceCache.bin"
	  Dim key() As Byte = Encrypto.TestHashKey

	  WriteCache(cacheFilePath, key)
	  ClearCache()
	  ReadCacheAndDisplayContents(cacheFilePath, key)

	  PromptToContinue()
	End Sub

	Private Sub ReadCacheAndDisplayContents(ByVal cacheFilePath As String, ByVal key() As Byte)
	  Console.WriteLine("Restoring cache state from {0}", cacheFilePath)
	  ImportCacheStateFile(cacheFilePath, key)
	  Console.WriteLine(vbLf & "Cache contents:" & vbLf)
	  DisplayCacheContents()
	End Sub

	Private Sub ClearCache()
	  Console.WriteLine(vbLf & "Clearing cache..." & vbLf)
	  _mgr.Clear()
	End Sub

	Private Sub WriteCache(ByVal cacheFilePath As String, ByVal key() As Byte)
	  Console.WriteLine("Saving encrypted cache state to {0}", cacheFilePath)
	  SaveCacheToCacheStateFile(cacheFilePath, key)
	End Sub

	Private Sub SaveCacheToCacheStateFile(ByVal cacheFilePath As String, ByVal key() As Byte)
	  Dim encryptedData() As Byte
	  Using cacheStateStream = New MemoryStream()
		_mgr.CacheStateManager.SaveCacheState(cacheStateStream, True)
		encryptedData = Encrypto.Encrypt(key, cacheStateStream)
		WriteByteArrayToFile(encryptedData, cacheFilePath)
	  End Using
	End Sub

	Private Sub ImportCacheStateFile(ByVal cacheFilePath As String, ByVal key() As Byte)
	  Dim encryptedData() As Byte
	  Dim decryptedData() As Byte

	  encryptedData = ReadByteArrayFromFile(cacheFilePath)
	  decryptedData = Encrypto.Decrypt(key, encryptedData)

	  Dim restoreSaveOptions As Boolean = True
	  Dim restoreQueryStrategy As Boolean = False
	  Dim cacheRestoreStrategy = New RestoreStrategy(restoreSaveOptions, restoreQueryStrategy, MergeStrategy.OverwriteChanges)

	  Try
		_mgr.CacheStateManager.RestoreCacheState(New MemoryStream(decryptedData), cacheRestoreStrategy, True)
	  Catch e As Exception
		Console.WriteLine(e.Message)
	  End Try
	End Sub


	Private Sub DoIt01()
	  LoadDataIntoCache()
	  Console.WriteLine("Entities present after initial retrieval: {0}", GetCountOfEntitiesInCache())
	  StoreCacheStateLocally()
	  _mgr.Clear()
	  Console.WriteLine("Entities present after Clear and encrypted local save: {0}", GetCountOfEntitiesInCache())
	  RestoreLocallyStoredCacheState()
	  Console.WriteLine("Entities present after restore of encrypted local file: {0}", GetCountOfEntitiesInCache())
	  PromptToContinue()
	End Sub

	Private _encryptionString As String = "EncryptMe"
	Private _encryptionKey() As Byte
	Private _targetPath As String = "c:\DevForceCache.bin"

	Private Sub LoadDataIntoCache()
	  Dim customers As New List(Of Customer)(_mgr.Customers.Include("Orders").Include("Orders.OrderDetails").ToList())
	End Sub

	Private Sub StoreCacheStateLocally()
	  _encryptionKey = CryptoFns.AesGetHashKey(_encryptionString)

	  Dim fipsCompliant As Boolean = True
	  Dim binaryCacheDataAsString As String = Nothing
	  Dim encryptedBinaryCacheDataAsString As String = Nothing
	  Dim aMemoryStream As New MemoryStream()
	  Dim aStreamWriter As New StreamWriter(_targetPath, False)

	  Try
		_mgr.CacheStateManager.SaveCacheState(aMemoryStream, True)
		binaryCacheDataAsString = Convert.ToBase64String(aMemoryStream.GetBuffer())
		encryptedBinaryCacheDataAsString = Crypto.AesEncrypt(binaryCacheDataAsString, _encryptionKey, fipsCompliant)

		' BEG Debugging variables
		Dim cacheString01 As String = binaryCacheDataAsString.Substring(0, 100)
		Dim encryptedCacheString01 As String = encryptedBinaryCacheDataAsString.Substring(0, 100)
		Dim decryptedCacheString01 As String = Crypto.AesDecrypt(encryptedBinaryCacheDataAsString, _encryptionKey, True).Substring(0, 100)
		' END Debugging variables

		aStreamWriter.Write(encryptedBinaryCacheDataAsString)
	  Catch ex As Exception
		Console.WriteLine(ex.Message)
	  Finally
		aStreamWriter.Flush()
		aStreamWriter.Close()
		Dim msg As String = "Local cache contents saved in encrypted form to local storage at: "
		Console.WriteLine(msg & _targetPath)
	  End Try
	End Sub

	Private Sub RestoreLocallyStoredCacheState()
	  _encryptionKey = CryptoFns.AesGetHashKey(_encryptionString)
	  Dim aMemoryStream As MemoryStream = Nothing
	  Dim encryptedBinaryCacheDataAsString As String = Nothing
	  Dim binaryCacheData() As Byte = Nothing

	  Try
		'Read the file
		Dim aStreamReader As New StreamReader(_targetPath)
		Try
		  'Convert the encrypted data to a string
		  encryptedBinaryCacheDataAsString = aStreamReader.ReadToEnd()

		  ' BEG debugging variables
		  Dim encryptedCacheString02 As String = encryptedBinaryCacheDataAsString.Substring(0, 100)
		  ' END debugging variables

		  'Decrypt the encrypted string
		  Dim decryptedBinaryCacheDataAsString As String = Crypto.AesDecrypt(encryptedBinaryCacheDataAsString, _encryptionKey, True)

		  ' BEG debugging variables
		  Dim decryptedCacheString02 As String = decryptedBinaryCacheDataAsString.Substring(0, 100)
		  ' END debugging variables

		  binaryCacheData = Convert.FromBase64String(decryptedBinaryCacheDataAsString)

		  'Read the decrypted string into a MemoryStream
		  aMemoryStream = New MemoryStream(binaryCacheData)
		  aMemoryStream.Position = 0
		  'Restore the EntitySet from the decrypted MemoryStream
		  _mgr.CacheStateManager.RestoreCacheState(aMemoryStream, RestoreStrategy.Normal, True)
		Catch ex As Exception
		  Console.WriteLine(ex.Message)
		Finally
		  'Close the reader and reload the Employee EntityList
		  aStreamReader.Close()
		  Dim employees As New List(Of Employee)(_mgr.Employees.ToList())
		End Try
	  Catch pException As Exception
		Console.WriteLine(pException.Message)
	  End Try
	End Sub

	#Region "Utility Methods"

	Private Function GetCountOfEntitiesInCache() As Integer
	  Return _mgr.Customers.With(QueryStrategy.CacheOnly).Count() + _mgr.Orders.With(QueryStrategy.CacheOnly).Count() + _mgr.OrderDetails.With(QueryStrategy.CacheOnly).Count()
	End Function

	Private Sub PromptToContinue()
	  Console.WriteLine()
	  Console.WriteLine("Press ENTER to continue...")
	  Console.ReadLine()
	End Sub

	Private Sub DisplayCacheContents()
	  Console.WriteLine("Employees: {0}", _mgr.Employees.With(QueryStrategy.CacheOnly).Count())
	  Console.WriteLine("Orders: {0}", _mgr.Orders.With(QueryStrategy.CacheOnly).Count())
	  Console.WriteLine("OrderDetails: {0}", _mgr.OrderDetails.With(QueryStrategy.CacheOnly).Count())
	  Console.WriteLine("Products: {0}", _mgr.Products.With(QueryStrategy.CacheOnly).Count())
	  Console.WriteLine("Suppliers: {0}", _mgr.Suppliers.With(QueryStrategy.CacheOnly).Count())
	  Console.WriteLine("Customers: {0}", _mgr.Customers.With(QueryStrategy.CacheOnly).Count())
	End Sub

	Public Function WriteByteArrayToFile(ByVal buff() As Byte, ByVal fileName As String) As Boolean
	  Dim response As Boolean = False

	  Try
		Dim fs As New FileStream(fileName, FileMode.Create, FileAccess.ReadWrite)
		Dim bw As New BinaryWriter(fs)
		bw.Write(buff)
		bw.Close() 'Thanks Karlo for pointing out!
		response = True
	  Catch ex As Exception
		Console.WriteLine(ex.Message)
	  End Try

	  Return response
	End Function

	Public Function ReadByteArrayFromFile(ByVal fileName As String) As Byte()
	  Dim buff() As Byte = Nothing
	  Dim fs As New FileStream(fileName, FileMode.Open, FileAccess.Read)
	  Dim br As New BinaryReader(fs)
	  Dim numBytes As Long = New FileInfo(fileName).Length
	  buff = br.ReadBytes(CInt(numBytes))
	  Return buff
	End Function

	#End Region ' Utility Methods

	#Region "Private Fields"
	Private _mgr As New NorthwindIBEntityManager()
	#End Region ' Private Fields
  End Class
End Namespace
