﻿using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;

namespace Console01 {

  /// <summary>
  /// Encryption helpers for EntityCacheState to/from isolated storage
  /// </summary>
  /// <remarks>
  /// Proof of concept implementation only. 
  /// No error handling.  
  /// Not efficient for large cache
  /// Hangs the thread on big cache files 
  /// Bad test key!
  /// </remarks>
  public static class Encrypto {

    private static byte[] _testHashKey;

    /// <summary>For test purposes only; do not use!</summary>
    /// <remarks>
    /// Aes key should never be stored on the client, neither in code nor as data
    /// For apps that might launch offline, best to build key from username+password
    /// </remarks>
    public static byte[] TestHashKey {
      get {
        if (null == _testHashKey) {
          _testHashKey = GetHashKey("Swordfish");
        }
        return _testHashKey;
      }
    }

    public static void EncryptToStream(byte[] key, MemoryStream streamToEncrypt, Stream outputStream) {

      var encryptor = new AesManaged {Key = key, IV = key};

      using (var encrypt =
        new CryptoStream(outputStream, encryptor.CreateEncryptor(), CryptoStreamMode.Write)) {
        // Encrypt
        byte[] dataToEncrypt = streamToEncrypt.ToArray();
        encrypt.Write(dataToEncrypt, 0, dataToEncrypt.Length);
        encrypt.FlushFinalBlock();
        encrypt.Close();
      }
    }

    public static byte[] Encrypt(byte[] key, MemoryStream streamToEncrypt) {

      using (var encryptionStream = new MemoryStream()) {
        EncryptToStream(key, streamToEncrypt, encryptionStream);
        return encryptionStream.ToArray();
      }

    }

    public static byte[] DecryptFromStream(byte[] key, Stream encryptedStream) {

      // Really dumb; should be async and sip the file; some error handling would be nice.
      var len =(int) encryptedStream.Length; // look out for big file!
      var encryptedData = new byte[len];
      encryptedStream.Read(encryptedData, 0, len);
      return Decrypt(key, encryptedData);
    }

    public static byte[] Decrypt(byte[] key, byte[] encryptedData) {

      var decryptor = new AesManaged {Key = key, IV = key};

      using (var decryptionStream = new MemoryStream()) {
        using (var decrypt = 
          new CryptoStream(decryptionStream, decryptor.CreateDecryptor(), CryptoStreamMode.Write)) {
          // Decrypt
          decrypt.Write(encryptedData, 0, encryptedData.Length);
          decrypt.Flush();
          decrypt.Close();

          return decryptionStream.ToArray();
        }
      }
    }

    #region Inspiration

    // Original http://silverlightuk.blogspot.com/2008/06/silverlight-encryption-part-2.html

    public static byte[] GetHashKey(string hashKey) {
      // Initialize
      UTF8Encoding encoder = new UTF8Encoding();

      // Get the salt
      string salt = "I am a nice little salt";
      byte[] saltBytes = encoder.GetBytes(salt);

      // Setup the hasher
      Rfc2898DeriveBytes rfc = new Rfc2898DeriveBytes(hashKey, saltBytes);

      // Return the key
      return rfc.GetBytes(16);
    }

    // Original Code from http://silverlightuk.blogspot.com/2008/06/silverlight-encryption-part-3.html

    public static string Encrypt(byte[] key, string dataToEncrypt) {
      // Initialize
      AesManaged encryptor = new AesManaged();

      // Set the key
      encryptor.Key = key;
      encryptor.IV = key;

      // create a memory stream
      using (MemoryStream encryptionStream = new MemoryStream()) {
        // Create the crypto stream
        using (CryptoStream encrypt = 
          new CryptoStream(encryptionStream, encryptor.CreateEncryptor(), CryptoStreamMode.Write)) {
          // Encrypt
          byte[] utfD1 = UTF8Encoding.UTF8.GetBytes(dataToEncrypt);
          encrypt.Write(utfD1, 0, utfD1.Length);
          encrypt.FlushFinalBlock();
          encrypt.Close();

          // Return the encrypted data
          return Convert.ToBase64String(encryptionStream.ToArray());
        }
      }
    }

    public static string Decrypt(byte[] key, string encryptedString) {
      // Initialise
      AesManaged decryptor = new AesManaged();
      byte[] encryptedData = Convert.FromBase64String(encryptedString);

      // Set the key
      decryptor.Key = key;
      decryptor.IV = key;

      // create a memory stream
      using (MemoryStream decryptionStream = new MemoryStream()) {
        // Create the crypto stream
        using (CryptoStream decrypt = new CryptoStream(decryptionStream, decryptor.CreateDecryptor(), CryptoStreamMode.Write)) {
          // Decrypt
          decrypt.Write(encryptedData, 0, encryptedData.Length);
          decrypt.Flush();
          decrypt.Close();

          // Return the unencrypted data
          byte[] decryptedData = decryptionStream.ToArray();
          return UTF8Encoding.UTF8.GetString(decryptedData, 0, decryptedData.Length);
        }
      }
    }

    #endregion

  }
}
