﻿Imports System.IO
Imports System.Security.Cryptography
Imports System.Text

Namespace Console01

  ''' <summary>
  ''' Encryption helpers for EntityCacheState to/from isolated storage
  ''' </summary>
  ''' <remarks>
  ''' Proof of concept implementation only. 
  ''' No error handling.  
  ''' Not efficient for large cache
  ''' Hangs the thread on big cache files 
  ''' Bad test key!
  ''' </remarks>
  Public NotInheritable Class Encrypto

	Private Shared _testHashKey() As Byte

	''' <summary>For test purposes only; do not use!</summary>
	''' <remarks>
	''' Aes key should never be stored on the client, neither in code nor as data
	''' For apps that might launch offline, best to build key from username+password
	''' </remarks>
	Private Sub New()
	End Sub
	Public Shared ReadOnly Property TestHashKey() As Byte()
	  Get
		If Nothing Is _testHashKey Then
		  _testHashKey = GetHashKey("Swordfish")
		End If
		Return _testHashKey
	  End Get
	End Property

	Public Shared Sub EncryptToStream(ByVal key() As Byte, ByVal streamToEncrypt As MemoryStream, ByVal outputStream As Stream)

	  Dim encryptor = New AesManaged With {.Key = key, .IV = key}

	  Using encrypt = New CryptoStream(outputStream, encryptor.CreateEncryptor(), CryptoStreamMode.Write)
		' Encrypt
		Dim dataToEncrypt() As Byte = streamToEncrypt.ToArray()
		encrypt.Write(dataToEncrypt, 0, dataToEncrypt.Length)
		encrypt.FlushFinalBlock()
		encrypt.Close()
	  End Using
	End Sub

	Public Shared Function Encrypt(ByVal key() As Byte, ByVal streamToEncrypt As MemoryStream) As Byte()

	  Using encryptionStream = New MemoryStream()
		EncryptToStream(key, streamToEncrypt, encryptionStream)
		Return encryptionStream.ToArray()
	  End Using

	End Function

	Public Shared Function DecryptFromStream(ByVal key() As Byte, ByVal encryptedStream As Stream) As Byte()

	  ' Really dumb; should be async and sip the file; some error handling would be nice.
	  Dim len =CInt(encryptedStream.Length) ' look out for big file!
	  Dim encryptedData = New Byte(len - 1){}
	  encryptedStream.Read(encryptedData, 0, len)
	  Return Decrypt(key, encryptedData)
	End Function

	Public Shared Function Decrypt(ByVal key() As Byte, ByVal encryptedData() As Byte) As Byte()

	  Dim decryptor = New AesManaged With {.Key = key, .IV = key}

	  Using decryptionStream = New MemoryStream()
        Using privatedecrypt = New CryptoStream(decryptionStream, decryptor.CreateDecryptor(), CryptoStreamMode.Write)
          ' Decrypt
          privatedecrypt.Write(encryptedData, 0, encryptedData.Length)
          privatedecrypt.Flush()
          privatedecrypt.Close()

          Return decryptionStream.ToArray()
        End Using
	  End Using
	End Function

	#Region "Inspiration"

	' Original http://silverlightuk.blogspot.com/2008/06/silverlight-encryption-part-2.html

	Public Shared Function GetHashKey(ByVal hashKey As String) As Byte()
	  ' Initialize
	  Dim encoder As New UTF8Encoding()

	  ' Get the salt
	  Dim salt As String = "I am a nice little salt"
	  Dim saltBytes() As Byte = encoder.GetBytes(salt)

	  ' Setup the hasher
	  Dim rfc As New Rfc2898DeriveBytes(hashKey, saltBytes)

	  ' Return the key
	  Return rfc.GetBytes(16)
	End Function

	' Original Code from http://silverlightuk.blogspot.com/2008/06/silverlight-encryption-part-3.html

	Public Shared Function Encrypt(ByVal key() As Byte, ByVal dataToEncrypt As String) As String
	  ' Initialize
	  Dim encryptor As New AesManaged()

	  ' Set the key
	  encryptor.Key = key
	  encryptor.IV = key

	  ' create a memory stream
	  Using encryptionStream As New MemoryStream()
		' Create the crypto stream
        Using privateencrypt As New CryptoStream(encryptionStream, encryptor.CreateEncryptor(), CryptoStreamMode.Write)
          ' Encrypt
          Dim utfD1() As Byte = UTF8Encoding.UTF8.GetBytes(dataToEncrypt)
          privateencrypt.Write(utfD1, 0, utfD1.Length)
          privateencrypt.FlushFinalBlock()
          privateencrypt.Close()

          ' Return the encrypted data
          Return Convert.ToBase64String(encryptionStream.ToArray())
        End Using
	  End Using
	End Function

	Public Shared Function Decrypt(ByVal key() As Byte, ByVal encryptedString As String) As String
	  ' Initialise
	  Dim decryptor As New AesManaged()
	  Dim encryptedData() As Byte = Convert.FromBase64String(encryptedString)

	  ' Set the key
	  decryptor.Key = key
	  decryptor.IV = key

	  ' create a memory stream
	  Using decryptionStream As New MemoryStream()
		' Create the crypto stream
        Using privatedecrypt As New CryptoStream(decryptionStream, decryptor.CreateDecryptor(), CryptoStreamMode.Write)
          ' Decrypt
          privatedecrypt.Write(encryptedData, 0, encryptedData.Length)
          privatedecrypt.Flush()
          privatedecrypt.Close()

          ' Return the unencrypted data
          Dim decryptedData() As Byte = decryptionStream.ToArray()
          Return UTF8Encoding.UTF8.GetString(decryptedData, 0, decryptedData.Length)
        End Using
	  End Using
	End Function

	#End Region

  End Class
End Namespace
